@@ -33,6 +33,7 @@ from cwl_airflow.utilities.cwl import (
     fast_cwl_load,
     slow_cwl_load,
     convert_to_workflow,
+    clean_up_dag_run,
     DAG_TEMPLATE
 )
 
@@ -75,7 +76,7 @@ class CWLApiBackend():
 
 
     def get_dags(self, dag_ids=[]):
-        logging.debug(f"Call get_dags with dag_ids={dag_ids}")
+        logging.info(f"Call get_dags with dag_ids={dag_ids}")
         try:
             dag_ids = dag_ids or self.list_dags()
             logging.debug(f"Processing dags {dag_ids}")
@@ -85,41 +86,39 @@ class CWLApiBackend():
             return {"dags": []}
 
 
-    def post_dag(self, dag_id=None):
-        logging.debug(f"Call post_dag with dag_id={dag_id}")
+    def post_dags(self, dag_id=None):
+        logging.info(f"Call post_dags with dag_id={dag_id}")
         try:
             res = self.export_dag(dag_id or ''.join(random.choice(string.ascii_lowercase) for i in range(32)))
             logging.debug(f"Exported DAG {res}")
             return res
         except Exception as err:
-            logging.error(f"Failed while running post_dag {err}")
+            logging.error(f"Failed while running post_dags {err}")
             return connexion.problem(500, "Failed to create dag", str(err))
 
 
     def get_dag_runs(self, dag_id=None, run_id=None, execution_date=None, state=None):
-        logging.debug(f"Call get_dag_runs with dag_id={dag_id}, run_id={run_id}, execution_date={execution_date}, state={state}")
+        logging.info(f"Call get_dag_runs with dag_id={dag_id}, run_id={run_id}, execution_date={execution_date}, state={state}")
         try:
             dag_runs = []
             dag_ids = [dag_id] if dag_id else self.list_dags()
-            logging.debug(f"Processing dags {dag_ids}")
+            logging.debug(f"Found dags {dag_ids}")
             for d_id in dag_ids:
-                logging.debug(f"Process dag  {d_id}")
-                task_ids = self.list_tasks(d_id)
-                logging.debug(f"Fetched tasks {task_ids}")
-                for dag_run in self.list_dag_runs(d_id, state):
-                    logging.debug(f"Process dag run {dag_run['run_id']}, {dag_run['execution_date']}")
-                    if run_id and run_id != dag_run["run_id"] or execution_date and execution_date != dag_run["execution_date"]:
-                        logging.debug(f"Skip dag_run {dag_run['run_id']}, {dag_run['execution_date']} (run_id or execution_date doesn't match)")
+                logging.info(f"Process dag  {d_id}")
+                for dag_run in DagRun.find(dag_id=d_id, state=state):
+                    logging.info(f"Process dag_run {dag_run.run_id}, {dag_run.execution_date.isoformat()}")
+                    if run_id and run_id != dag_run.run_id or execution_date and execution_date != dag_run.execution_date.isoformat():
+                        logging.info(f"Skip dag_run {dag_run.run_id}, {dag_run.execution_date.isoformat()} (run_id or execution_date doesn't match)")
                         continue
-                    response_item = {"dag_id": d_id,
-                                     "run_id": dag_run["run_id"],
-                                     "execution_date": dag_run["execution_date"],
-                                     "start_date": dag_run["start_date"],
-                                     "state": dag_run["state"],
-                                     "tasks": []}
-                    logging.debug(f"Get statuses for tasks {task_ids}")
-                    for t_id in task_ids:
-                        response_item["tasks"].append({"id": t_id, "state": self.task_state(d_id, t_id, dag_run["execution_date"])})
+                    response_item = {
+                        "dag_id": d_id,
+                        "run_id": dag_run.run_id,
+                        "execution_date": dag_run.execution_date.isoformat(),
+                        "start_date": dag_run.start_date.isoformat(),
+                        "state": dag_run.state,
+                        "tasks": [{"id": ti.task_id, "state": ti.state} for ti in dag_run.get_task_instances()],
+                        "progress": int(len([ti for ti in dag_run.get_task_instances(State.SUCCESS)]) / len(dag_run.get_task_instances()) * 100)
+                    }
                     dag_runs.append(response_item)
             return {"dag_runs": dag_runs}
         except Exception as err:
@@ -128,7 +127,7 @@ class CWLApiBackend():
         
 
     def post_dag_runs(self, dag_id, run_id=None, conf=None):
-        logging.debug(f"Call post_dag_runs with dag_id={dag_id}, run_id={run_id}, conf={conf}")
+        logging.info(f"Call post_dag_runs with dag_id={dag_id}, run_id={run_id}, conf={conf}")
         try:
             dagrun = self.trigger_dag(dag_id, run_id, conf)
             return {"dag_id": dagrun.dag_id,
@@ -141,9 +140,20 @@ class CWLApiBackend():
             return connexion.problem(500, "Failed to create dag_run", str(err))
 
 
+    def post_dags_dag_runs(self, dag_id, run_id, conf=None):
+        logging.info(f"Call post_dags_dag_runs with dag_id={dag_id}, run_id={run_id}, conf={conf}")
+        self.post_dags(dag_id)
+        clean_up_dag_run(
+            dag_id=dag_id,
+            run_id=run_id,
+            kill_timeout=3  # use shorter timeout for killing runnign tasks
+        )                   # should wait untill it finish running, as we can't trigger the same DAG with the same run_id
+        return self.post_dag_runs(dag_id, run_id, conf)
+
+
     def post_dag_runs_legacy(self, dag_id):
         data = connexion.request.json
-        logging.debug(f"Call post_dag_runs_legacy with dag_id={dag_id}, data={data}")
+        logging.info(f"Call post_dag_runs_legacy with dag_id={dag_id}, data={data}")
         return self.post_dag_runs(dag_id, data["run_id"], data["conf"])
 
 
@@ -223,18 +233,6 @@ class CWLApiBackend():
         return task_state
 
 
-    def list_dag_runs(self, dag_id, state):
-        dag_runs = []
-        for dag_run in DagRun.find(dag_id=dag_id, state=state):
-            dag_runs.append({
-                "run_id": dag_run.run_id,
-                "state": dag_run.state,
-                "execution_date": dag_run.execution_date.isoformat(),
-                "start_date": ((dag_run.start_date or '') and dag_run.start_date.isoformat())
-            })
-        return dag_runs
-
-
     def save_attachment(self, attachment, location, exist_ok=False):
         if path.isfile(location) and not exist_ok:
             raise FileExistsError(f"File {location} already exist")
