@@ -167,6 +167,12 @@ def get_parser():
         help="Embed base64 encoded zlib compressed workflow content into the DAG python file. \
             Default: False"
     )
+    test_parser.add_argument(
+        "--combine",
+        action="store_true",
+        help="Create and trigger DAG using a single API call. \
+            Default: False"
+    )
 
     return general_parser
 
@@ -246,7 +252,7 @@ def parse_arguments(argsl, cwd=None):
     args, _ = get_parser().parse_known_args(argsl)
     args = get_normalized_args(
         args,
-        ["func", "port", "host", "api", "range", "spin", "embed", "upgrade"],
+        ["func", "port", "host", "api", "range", "spin", "embed", "upgrade", "combine"],
         cwd
     )
     assert_and_fix_args(args)
